@@ -12,7 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import distutils
 import os
 import pytest
 
@@ -62,15 +61,15 @@ def test_nonslash_resource(messaging):
 
 
 def test_path_parsing(messaging):
-    expected = {"room_id": "tiki"}
+    expected = {"room": "tiki"}
     actual = messaging.parse_room_path(messaging.room_path("tiki"))
 
     assert expected == actual
 
     expected = {
-        "user_id": "bdfl",
-        "legacy_user_id": "apocalyptic",
-        "blurb_id": "city",
+        "user": "bdfl",
+        "legacy_user": "apocalyptic",
+        "blurb": "city",
     }
     actual = messaging.parse_blurb_path(
         messaging.blurb_path("bdfl", "apocalyptic", "city")
@@ -78,7 +77,7 @@ def test_path_parsing(messaging):
     assert expected == actual
 
 
-if distutils.util.strtobool(os.environ.get("GAPIC_PYTHON_ASYNC", "true")):
+if os.environ.get("GAPIC_PYTHON_ASYNC", "true") == "true":
 
     @pytest.mark.asyncio
     async def test_crud_with_request_async(async_identity):
