@@ -14,6 +14,7 @@
 
 import os
 import pytest
+import re
 from unittest import mock
 import warnings
 
@@ -139,16 +140,71 @@ def test_options_service_config(fs):
     assert opts.retry == expected_cfg
 
 
-def test_options_lazy_import():
-    opts = Options.build('lazy-import')
-    assert opts.lazy_import
+def test_options_service_yaml_config(fs):
+    opts = Options.build("")
+    assert opts.service_yaml_config == {}
+
+    service_yaml_fpath = "testapi_v1.yaml"
+    fs.create_file(service_yaml_fpath,
+                   contents=("type: google.api.Service\n"
+                             "config_version: 3\n"
+                             "name: testapi.googleapis.com\n"))
+    opt_string = f"service-yaml={service_yaml_fpath}"
+    opts = Options.build(opt_string)
+    expected_config = {
+        "config_version": 3,
+        "name": "testapi.googleapis.com"
+    }
+    assert opts.service_yaml_config == expected_config
+
+    service_yaml_fpath = "testapi_v2.yaml"
+    fs.create_file(service_yaml_fpath,
+                   contents=("config_version: 3\n"
+                             "name: testapi.googleapis.com\n"))
+    opt_string = f"service-yaml={service_yaml_fpath}"
+    opts = Options.build(opt_string)
+    expected_config = {
+        "config_version": 3,
+        "name": "testapi.googleapis.com"
+    }
+    assert opts.service_yaml_config == expected_config
 
 
-def test_options_old_naming():
-    opts = Options.build('old-naming')
-    assert opts.old_naming
+def test_options_bool_flags():
+    # Most options are default False.
+    # New options should follow the dash-case/snake_case convention.
+    opt_str_to_attr_name = {
+        name: re.sub(r"-", "_", name)
+        for name in
+        ["lazy-import",
+         "old-naming",
+         "add-iam-methods",
+         "metadata",
+         "warehouse-package-name",
+         ]}
+
+    for opt, attr in opt_str_to_attr_name.items():
+        options = Options.build("")
+        assert not getattr(options, attr)
+
+        options = Options.build(opt)
+        assert getattr(options, attr)
+
+    # Check autogen-snippets separately, as it is default True
+    options = Options.build("")
+    assert options.autogen_snippets
+
+    options = Options.build("autogen-snippets=False")
+    assert not options.autogen_snippets
 
 
-def test_options_add_iam_methods():
-    opts = Options.build('add-iam-methods')
-    assert opts.add_iam_methods
+def test_options_autogen_snippets_false_for_old_naming():
+    # NOTE: Snippets are not currently correct for the alternative (Ads) templates
+    # so always disable snippetgen in that case
+    # https://github.com/googleapis/gapic-generator-python/issues/1052
+    options = Options.build("old-naming")
+    assert not options.autogen_snippets
+
+    # Even if autogen-snippets is set to True, do not enable snippetgen
+    options = Options.build("old-naming,autogen-snippets=True")
+    assert not options.autogen_snippets
