@@ -15,14 +15,20 @@
 import jinja2
 import os.path as path
 import pytest
+from pathlib import Path
+
+from google.protobuf import json_format
 
 import gapic.utils as utils
 
 from gapic.samplegen import samplegen
 from gapic.samplegen_utils import (types, utils as gapic_utils)
+from gapic.samplegen_utils import snippet_metadata_pb2
 from gapic.schema import (naming, wrappers)
 
-from common_types import (DummyField, DummyMessage, DummyMethod, DummyService,
+from ..common_types import (DummyField, DummyMessage,
+
+                          DummyMessageTypePB, DummyMethod, DummyService, DummyIdent,
                           DummyApiSchema, DummyNaming, enum_factory, message_factory)
 
 from collections import namedtuple
@@ -41,6 +47,13 @@ env = jinja2.Environment(
 )
 env.filters['snake_case'] = utils.to_snake_case
 env.filters['coerce_response_name'] = gapic_utils.coerce_response_name
+env.filters['render_format_string'] = gapic_utils.render_format_string
+
+
+def golden_snippet(filename: str) -> str:
+    """Load the golden snippet with the name provided"""
+    snippet_path = Path(__file__).parent / "golden_snippets" / filename
+    return snippet_path.read_text()
 
 
 def test_generate_sample_basic():
@@ -49,23 +62,43 @@ def test_generate_sample_basic():
     # that catch errors in behavior that is emergent from combining smaller features
     # or in features that are sufficiently small and trivial that it doesn't make sense
     # to have standalone tests.
+
+    classify_target_field = DummyField(
+        name="classify_target",
+        type=DummyMessageTypePB(name="ClassifyTarget"),
+        message=DummyMessage(
+            type="CLASSIFY TYPE",
+            fields={
+                "video": DummyField(
+                    type=DummyMessageTypePB(name="Video"),
+                    message=DummyMessage(type="VIDEO TYPE"),
+                ),
+                "location_annotation": DummyField(
+                    type=DummyMessageTypePB(name="Location"),
+                    message=DummyMessage(type="LOCATION TYPE"),
+                )
+            },
+        ),
+        ident=DummyIdent(sphinx="molluscs_v1.ClassifyTarget")
+    )
+
     input_type = DummyMessage(
         type="REQUEST TYPE",
         fields={
-            "classify_target": DummyField(
-                message=DummyMessage(
-                    type="CLASSIFY TYPE",
-                    fields={
-                        "video": DummyField(
-                            message=DummyMessage(type="VIDEO TYPE"),
-                        ),
-                        "location_annotation": DummyField(
-                            message=DummyMessage(type="LOCATION TYPE"),
-                        )
-                    },
-                )
+            "classify_target": classify_target_field
+        },
+        ident=DummyIdent(name="molluscs.v1.ClassifyRequest",
+                         sphinx="molluscs_v1.classify_request")
+    )
+
+    output_type = DummyMessage(
+        type="RESPONSE TYPE",
+        fields={
+            "classification": DummyField(
+                type=DummyMessageTypePB(name="Classification"),
             )
-        }
+        },
+        ident=DummyIdent(sphinx="molluscs_v1.classification")
     )
 
     api_naming = naming.NewNaming(
@@ -74,10 +107,12 @@ def test_generate_sample_basic():
         service_pb=namedtuple('service_pb', ['name'])('MolluscService'),
         methods={
             "Classify": DummyMethod(
+                name="Classify",
                 input=input_type,
                 output=message_factory("$resp.taxonomy"),
+                client_output=output_type,
                 flattened_fields={
-                    "classify_target": DummyField(name="classify_target")
+                    "classify_target": classify_target_field
                 }
             )
         },
@@ -90,6 +125,7 @@ def test_generate_sample_basic():
     )
 
     sample = {"service": "animalia.mollusca.v1.Mollusc",
+              "region_tag": "molluscs_generated_molluscs_v1_Mollusc_Classify_sync",
               "rpc": "Classify",
               "id": "mollusc_classify_sync",
               "description": "Determine the full taxonomy of input mollusc",
@@ -104,83 +140,169 @@ def test_generate_sample_basic():
               ],
               "response": [{"print": ['Mollusc is a "%s"', "$resp.taxonomy"]}]}
 
-    sample_str = samplegen.generate_sample(
+    sample_str, metadata = samplegen.generate_sample(
         sample,
         schema,
         env.get_template('examples/sample.py.j2')
     )
 
-    sample_id = ("mollusc_classify_sync")
-    expected_str = '''# -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-#
-# DO NOT EDIT! This is a generated sample ("request",  "%s")
-#
-# To install the latest published package dependency, execute the following:
-#   pip3 install molluscs-v1-molluscclient
+    assert sample_str == golden_snippet("sample_basic.py")
+
+    assert json_format.MessageToDict(metadata) == {
+        'regionTag': 'molluscs_generated_molluscs_v1_Mollusc_Classify_sync',
+        'description': 'Sample for Classify',
+        'language': 'PYTHON',
+        'clientMethod': {
+            'shortName': 'classify',
+            'fullName': 'molluscs.v1.molluscclient.MolluscServiceClient.classify',
+            'parameters': [
+                {'type': 'molluscs_v1.classify_request', 'name': 'request'},
+                {'type': 'molluscs_v1.ClassifyTarget', 'name': 'classify_target'},
+                {'type': 'google.api_core.retry.Retry', 'name': 'retry'},
+                {'type': 'float', 'name': 'timeout'},
+                {'type': 'Sequence[Tuple[str, str]', 'name': 'metadata'}
+            ],
+            'resultType': 'molluscs_v1.classification',
+            'client': {
+                'shortName': 'MolluscServiceClient',
+                'fullName': 'molluscs.v1.molluscclient.MolluscServiceClient'
+            },
+            'method': {
+                'shortName': 'Classify',
+                'fullName': '.MolluscService.Classify',
+                'service': {'shortName': 'MolluscService', 'fullName': '.MolluscService'}}
+            },
+        'canonical': True,
+        'origin': 'API_DEFINITION'
+    }
 
 
-# [START %s]
-from google import auth
-from google.auth import credentials
-from molluscs.v1.molluscclient.services.mollusc_service import MolluscServiceClient
+def test_generate_sample_basic_async():
+    # Note: the sample integration tests are needfully large
+    # and difficult to eyeball parse. They are intended to be integration tests
+    # that catch errors in behavior that is emergent from combining smaller features
+    # or in features that are sufficiently small and trivial that it doesn't make sense
+    # to have standalone tests.
 
-def sample_classify(video, location):
-    """Determine the full taxonomy of input mollusc"""
-
-    client = MolluscServiceClient(
-        credentials=credentials.AnonymousCredentials(),
-        transport="grpc",
+    classify_target_field = DummyField(
+        name="classify_target",
+        type=DummyMessageTypePB(name="ClassifyTarget"),
+        message=DummyMessage(
+            type="CLASSIFY TYPE",
+            fields={
+                "video": DummyField(
+                    type=DummyMessageTypePB(name="Video"),
+                    message=DummyMessage(type="VIDEO TYPE"),
+                ),
+                "location_annotation": DummyField(
+                    type=DummyMessageTypePB(name="Location"),
+                    message=DummyMessage(type="LOCATION TYPE"),
+                )
+            },
+        ),
+        ident=DummyIdent(sphinx="molluscs_v1.ClassifyTarget")
     )
 
-    classify_target = {}
-    # video = "path/to/mollusc/video.mkv"
-    with open(video, "rb") as f:
-        classify_target["video"] = f.read()
+    input_type = DummyMessage(
+        type="REQUEST TYPE",
+        fields={
+            "classify_target": classify_target_field
+        },
+        ident=DummyIdent(name="molluscs.v1.ClassifyRequest",
+                         sphinx="molluscs_v1.classify_request")
+    )
 
-    # location = "New Zealand"
-    classify_target["location_annotation"] = location
+    output_type = DummyMessage(
+        type="RESPONSE TYPE",
+        fields={
+            "classification": DummyField(
+                type=DummyMessageTypePB(name="Classification"),
+            )
+        },
+        ident=DummyIdent(sphinx="molluscs_v1.classification")
+    )
 
- 
- 
-    response = client.classify(classify_target=classify_target)
-    print("Mollusc is a \\"{}\\"".format(response.taxonomy))
- 
+    api_naming = naming.NewNaming(
+        name="MolluscClient", namespace=("molluscs", "v1"))
+    service = wrappers.Service(
+        service_pb=namedtuple('service_pb', ['name'])('MolluscService'),
+        methods={
+            "Classify": DummyMethod(
+                name="Classify",
+                input=input_type,
+                output=message_factory("$resp.taxonomy"),
+                client_output_async=output_type,
+                client_output=output_type,
+                flattened_fields={
+                    "classify_target": classify_target_field
+                }
+            )
+        },
+        visible_resources={},
+    )
 
-# [END %s]
+    schema = DummyApiSchema(
+        services={"animalia.mollusca.v1.Mollusc": service},
+        naming=api_naming,
+    )
 
-def main():
-    import argparse
+    sample = {"service": "animalia.mollusca.v1.Mollusc",
+              "region_tag": "molluscs_generated_molluscs_v1_Mollusc_Classify_async",
+              "rpc": "Classify",
+              "transport": "grpc-async",
+              "id": "mollusc_classify_sync",
+              "description": "Determine the full taxonomy of input mollusc",
+              "request": [
+                  {"field": "classify_target.video",
+                   "value": "path/to/mollusc/video.mkv",
+                   "input_parameter": "video",
+                   "value_is_file": True},
+                  {"field": "classify_target.location_annotation",
+                   "value": "New Zealand",
+                   "input_parameter": "location"}
+              ],
+              "response": [{"print": ['Mollusc is a "%s"', "$resp.taxonomy"]}]}
 
-    parser = argparse.ArgumentParser()
-    parser.add_argument("--video",
-                        type=str,
-                        default="path/to/mollusc/video.mkv")
-    parser.add_argument("--location",
-                        type=str,
-                        default="New Zealand")
-    args = parser.parse_args()
+    sample_str, metadata = samplegen.generate_sample(
+        sample,
+        schema,
+        env.get_template('examples/sample.py.j2')
+    )
 
-    sample_classify(args.video, args.location)
+    assert sample_str == golden_snippet("sample_basic_async.py")
 
-
-if __name__ == "__main__":
-    main()
-''' % (sample_id, sample_id, sample_id)
-
-    assert sample_str == expected_str
+    assert json_format.MessageToDict(metadata) == {
+        'regionTag': 'molluscs_generated_molluscs_v1_Mollusc_Classify_async',
+        'description': 'Sample for Classify',
+        'language': 'PYTHON',
+        'clientMethod': {
+            'shortName': 'classify',
+            'fullName': 'molluscs.v1.molluscclient.MolluscServiceAsyncClient.classify',
+            'async': True,
+            'parameters': [
+                {'type': 'molluscs_v1.classify_request', 'name': 'request'},
+                {'type': 'molluscs_v1.ClassifyTarget', 'name': 'classify_target'},
+                {'type': 'google.api_core.retry.Retry', 'name': 'retry'},
+                {'type': 'float', 'name': 'timeout'},
+                {'type': 'Sequence[Tuple[str, str]', 'name': 'metadata'}
+            ],
+            'resultType': 'molluscs_v1.classification',
+            'client': {
+                'shortName': 'MolluscServiceAsyncClient',
+                'fullName': 'molluscs.v1.molluscclient.MolluscServiceAsyncClient'
+            },
+            'method': {
+                'shortName': 'Classify',
+                'fullName': '.MolluscService.Classify',
+                'service': {
+                    'shortName': 'MolluscService',
+                    'fullName': '.MolluscService'
+                }
+            }
+        },
+        'canonical': True,
+        'origin': 'API_DEFINITION'
+    }
 
 
 def test_generate_sample_basic_unflattenable():
@@ -193,19 +315,33 @@ def test_generate_sample_basic_unflattenable():
         type="REQUEST TYPE",
         fields={
             "classify_target": DummyField(
+                type=DummyMessageTypePB(name="ClassifyTarget"),
                 message=DummyMessage(
-                    type="CLASSIFY TYPE",
                     fields={
                         "video": DummyField(
+                            type=DummyMessageTypePB(name="Video"),
                             message=DummyMessage(type="VIDEO TYPE"),
                         ),
                         "location_annotation": DummyField(
+                            type=DummyMessageTypePB(name="Location"),
                             message=DummyMessage(type="LOCATION TYPE"),
                         )
                     },
                 )
             )
-        }
+        },
+        ident=DummyIdent(name="molluscs.v1.ClassifyRequest",
+                         sphinx="molluscs_v1.classify_request")
+    )
+
+    output_type = DummyMessage(
+        type="RESPONSE TYPE",
+        fields={
+            "classification": DummyField(
+                type=DummyMessageTypePB(name="Classification"),
+            )
+        },
+        ident=DummyIdent(sphinx="molluscs_v1.classification")
     )
 
     api_naming = naming.NewNaming(
@@ -214,8 +350,10 @@ def test_generate_sample_basic_unflattenable():
         service_pb=namedtuple('service_pb', ['name'])('MolluscService'),
         methods={
             "Classify": DummyMethod(
+                name="Classify",
                 input=input_type,
                 output=message_factory("$resp.taxonomy"),
+                client_output=output_type,
             )
         },
         visible_resources={},
@@ -227,6 +365,7 @@ def test_generate_sample_basic_unflattenable():
     )
 
     sample = {"service": "animalia.mollusca.v1.Mollusc",
+              "region_tag": "molluscs_generated_molluscs_v1_Mollusc_Classify_sync",
               "rpc": "Classify",
               "id": "mollusc_classify_sync",
               "description": "Determine the full taxonomy of input mollusc",
@@ -241,86 +380,146 @@ def test_generate_sample_basic_unflattenable():
               ],
               "response": [{"print": ['Mollusc is a "%s"', "$resp.taxonomy"]}]}
 
-    sample_str = samplegen.generate_sample(
+    sample_str, metadata = samplegen.generate_sample(
         sample,
         schema,
         env.get_template('examples/sample.py.j2')
     )
 
-    sample_id = ("mollusc_classify_sync")
-    expected_str = '''# -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-#
-# DO NOT EDIT! This is a generated sample ("request",  "%s")
-#
-# To install the latest published package dependency, execute the following:
-#   pip3 install molluscs-v1-molluscclient
+    assert sample_str == golden_snippet("sample_basic_unflattenable.py")
+
+    assert json_format.MessageToDict(metadata) == {
+        'regionTag': 'molluscs_generated_molluscs_v1_Mollusc_Classify_sync',
+        'description': 'Sample for Classify',
+        'language': 'PYTHON',
+        'clientMethod': {
+            'shortName': 'classify',
+            'fullName': 'molluscs.v1.molluscclient.MolluscServiceClient.classify',
+            'parameters': [
+                {'type': 'molluscs_v1.classify_request', 'name': 'request'},
+                {'type': 'google.api_core.retry.Retry', 'name': 'retry'},
+                {'type': 'float', 'name': 'timeout'},
+                {'type': 'Sequence[Tuple[str, str]', 'name': 'metadata'}
+            ],
+            'resultType': 'molluscs_v1.classification',
+            'client': {
+                'shortName': 'MolluscServiceClient',
+                'fullName': 'molluscs.v1.molluscclient.MolluscServiceClient'
+            },
+            'method': {
+                'shortName': 'Classify',
+                'fullName': '.MolluscService.Classify',
+                'service': {'shortName': 'MolluscService', 'fullName': '.MolluscService'}
+            }
+        },
+        'canonical': True,
+        'origin': 'API_DEFINITION'
+    }
 
 
-# [START %s]
-from google import auth
-from google.auth import credentials
-from molluscs.v1.molluscclient.services.mollusc_service import MolluscServiceClient
-
-def sample_classify(video, location):
-    """Determine the full taxonomy of input mollusc"""
-
-    client = MolluscServiceClient(
-        credentials=credentials.AnonymousCredentials(),
-        transport="grpc",
+def test_generate_sample_void_method():
+    classify_target_field = DummyField(
+        name="classify_target",
+        type=DummyMessageTypePB(name="ClassifyTarget"),
+        message=DummyMessage(
+            type="CLASSIFY TYPE",
+            fields={
+                "video": DummyField(
+                    type=DummyMessageTypePB(name="Video"),
+                    message=DummyMessage(type="VIDEO TYPE"),
+                ),
+                "location_annotation": DummyField(
+                    type=DummyMessageTypePB(name="Location"),
+                    message=DummyMessage(type="LOCATION TYPE"),
+                )
+            },
+        ),
+        ident=DummyIdent(sphinx="molluscs_v1.ClassifyTarget")
     )
 
-    classify_target = {}
-    # video = "path/to/mollusc/video.mkv"
-    with open(video, "rb") as f:
-        classify_target["video"] = f.read()
+    input_type = DummyMessage(
+        type="REQUEST TYPE",
+        fields={
+            "classify_target": classify_target_field
+        },
+        ident=DummyIdent(name="molluscs.v1.ClassifyRequest",
+                         sphinx="molluscs_v1.classify_request")
+    )
 
-    # location = "New Zealand"
-    classify_target["location_annotation"] = location
+    api_naming = naming.NewNaming(
+        name="MolluscClient", namespace=("molluscs", "v1"))
+    service = wrappers.Service(
+        service_pb=namedtuple('service_pb', ['name'])('MolluscService'),
+        methods={
+            "Classify": DummyMethod(
+                name="Classify",
+                client_output=DummyIdent(name="classify", sphinx="classify"),
+                void=True,
+                input=input_type,
+                output=message_factory("$resp.taxonomy"),
+                flattened_fields={
+                    "classify_target": classify_target_field,
+                }
+            )
+        },
+        visible_resources={},
+    )
 
-    request = {
-        'classify_target': classify_target,
+    schema = DummyApiSchema(
+        services={"animalia.mollusca.v1.Mollusc": service},
+        naming=api_naming,
+    )
+
+    sample = {"service": "animalia.mollusca.v1.Mollusc",
+              "region_tag": "molluscs_generated_molluscs_v1_Mollusc_Classify_sync",
+              "rpc": "Classify",
+              "id": "mollusc_classify_sync",
+              "description": "Determine the full taxonomy of input mollusc",
+              "request": [
+                  {"field": "classify_target.video",
+                   "value": "path/to/mollusc/video.mkv",
+                   "input_parameter": "video",
+                   "value_is_file": True},
+                  {"field": "classify_target.location_annotation",
+                   "value": "New Zealand",
+                   "input_parameter": "location"}
+              ]}
+
+    sample_str, metadata = samplegen.generate_sample(
+        sample,
+        schema,
+        env.get_template('examples/sample.py.j2')
+    )
+
+    assert sample_str == golden_snippet("sample_basic_void_method.py")
+
+    assert json_format.MessageToDict(metadata) == {
+        'regionTag': 'molluscs_generated_molluscs_v1_Mollusc_Classify_sync',
+        'description': 'Sample for Classify',
+        'language': 'PYTHON',
+        'clientMethod': {
+            'shortName': 'classify',
+            'fullName': 'molluscs.v1.molluscclient.MolluscServiceClient.classify',
+            'parameters': [
+                {'type': 'molluscs_v1.classify_request', 'name': 'request'},
+                {'type': 'molluscs_v1.ClassifyTarget', 'name': 'classify_target'},
+                {'type': 'google.api_core.retry.Retry', 'name': 'retry'},
+                {'type': 'float', 'name': 'timeout'},
+                {'type': 'Sequence[Tuple[str, str]', 'name': 'metadata'}
+            ],
+            'client': {
+                'shortName': 'MolluscServiceClient',
+                'fullName': 'molluscs.v1.molluscclient.MolluscServiceClient'
+            },
+            'method': {
+                'shortName': 'Classify',
+                'fullName': '.MolluscService.Classify',
+                'service': {'shortName': 'MolluscService', 'fullName': '.MolluscService'}
+            }
+        },
+        'canonical': True,
+        'origin': 'API_DEFINITION'
     }
- 
- 
-    response = client.classify(request=request)
-    print("Mollusc is a \\"{}\\"".format(response.taxonomy))
- 
-
-# [END %s]
-
-def main():
-    import argparse
-
-    parser = argparse.ArgumentParser()
-    parser.add_argument("--video",
-                        type=str,
-                        default="path/to/mollusc/video.mkv")
-    parser.add_argument("--location",
-                        type=str,
-                        default="New Zealand")
-    args = parser.parse_args()
-
-    sample_classify(args.video, args.location)
-
-
-if __name__ == "__main__":
-    main()
-''' % (sample_id, sample_id, sample_id)
-
-    assert sample_str == expected_str
 
 
 def test_generate_sample_service_not_found():
@@ -337,7 +536,7 @@ def test_generate_sample_service_not_found():
 
 def test_generate_sample_rpc_not_found():
     schema = DummyApiSchema(
-        {"Mollusc": DummyService({})}, DummyNaming("pkg_name"))
+        {"Mollusc": DummyService(methods={}, client_name="ClassifyClient")}, DummyNaming("pkg_name"))
     sample = {"service": "Mollusc", "rpc": "Classify"}
 
     with pytest.raises(types.RpcMethodNotFound):
