@@ -21,6 +21,7 @@ import gapic.utils as utils
 
 from gapic.samplegen_utils.types import CallingForm
 from textwrap import dedent
+from .. import common_types
 
 
 def check_template(template_fragment, expected_output, **kwargs):
@@ -48,6 +49,7 @@ def check_template(template_fragment, expected_output, **kwargs):
 
     env.filters['snake_case'] = utils.to_snake_case
     env.filters['coerce_response_name'] = sample_utils.coerce_response_name
+    env.filters['render_format_string'] = sample_utils.render_format_string
 
     template = env.get_template("template_fragment")
     text = template.render(**kwargs)
@@ -63,7 +65,7 @@ def test_render_attr_value():
         {{ frags.render_request_attr("mollusc", request) }}
         ''',
         '''
-        mollusc["order"] = Molluscs.Cephalopoda.Coleoidea
+        mollusc.order = Molluscs.Cephalopoda.Coleoidea
         ''',
         request=samplegen.AttributeRequestSetup(
             field="order",
@@ -79,8 +81,9 @@ def test_render_attr_input_parameter():
         {{ frags.render_request_attr("squid", request) }}
         ''',
         '''
+
         # species = 'Humboldt'
-        squid["species"] = species
+        squid.species = species
         ''',
         request=samplegen.AttributeRequestSetup(field="species",
                                                 value="'Humboldt'",
@@ -94,9 +97,10 @@ def test_render_attr_file():
         {{ frags.render_request_attr("classify_mollusc_request", request) }}
         ''',
         '''
+
         # mollusc_video_path = 'path/to/mollusc/video.mkv'
         with open(mollusc_video_path, "rb") as f:
-            classify_mollusc_request["mollusc_video"] = f.read()
+            classify_mollusc_request.mollusc_video = f.read()
         ''',
         request=samplegen.AttributeRequestSetup(field="mollusc_video",
                                                 value="'path/to/mollusc/video.mkv'",
@@ -109,28 +113,29 @@ def test_render_request_basic():
     check_template(
         '''
         {% import "feature_fragments.j2" as frags %}
-        {{ frags.render_request_setup(request) }}
+        {{ frags.render_request_setup(request, module_name, request_type) }}
         ''',
         '''
-        cephalopod = {}
+        # Initialize request argument(s)
+        cephalopod = mollusca.Cephalopod()
+
         # cephalopod_mass = '10 kg'
-        cephalopod["mantle_mass"] = cephalopod_mass
+        cephalopod.mantle_mass = cephalopod_mass
 
         # photo_path = 'path/to/cephalopod/photo.jpg'
         with open(photo_path, "rb") as f:
-            cephalopod["photo"] = f.read()
+            cephalopod.photo = f.read()
+        cephalopod.order = Molluscs.Cephalopoda.Coleoidea
 
-        cephalopod["order"] = Molluscs.Cephalopoda.Coleoidea
+        gastropod = mollusca.Gastropod()
 
-        gastropod = {}
         # gastropod_mass = '1 kg'
-        gastropod["mantle_mass"] = gastropod_mass
-
-        gastropod["order"] = Molluscs.Gastropoda.Pulmonata
+        gastropod.mantle_mass = gastropod_mass
+        gastropod.order = Molluscs.Gastropoda.Pulmonata
 
         # movie_path = 'path/to/gastropod/movie.mkv'
         with open(movie_path, "rb") as f:
-            gastropod["movie"] = f.read()
+            gastropod.movie = f.read()
 
         ''',
         request=samplegen.FullRequest(
@@ -174,6 +179,19 @@ def test_render_request_basic():
                                              single=None),
             ],
             flattenable=True,
+        ),
+        module_name="mollusca",
+        request_type=common_types.DummyMessage(
+            fields={
+                "cephalopod": common_types.DummyField(
+                    name="cephalopod",
+                    type=common_types.DummyMessageTypePB(name="Cephalopod")
+                ),
+                "gastropod": common_types.DummyField(
+                    name="gastropod",
+                    type=common_types.DummyMessageTypePB(name="Gastropod")
+                )
+            }
         )
     )
 
@@ -182,34 +200,35 @@ def test_render_request_unflattened():
     check_template(
         '''
         {% import "feature_fragments.j2" as frags %}
-        {{ frags.render_request_setup(request) }}
+        {{ frags.render_request_setup(request, module_name, request_type, calling_form, calling_form_enum) }}
         ''',
         '''
-        cephalopod = {}
+        # Initialize request argument(s)
+        cephalopod = mollusca.Cephalopod()
+
         # cephalopod_mass = '10 kg'
-        cephalopod["mantle_mass"] = cephalopod_mass
+        cephalopod.mantle_mass = cephalopod_mass
 
         # photo_path = 'path/to/cephalopod/photo.jpg'
         with open(photo_path, "rb") as f:
-            cephalopod["photo"] = f.read()
+            cephalopod.photo = f.read()
+        cephalopod.order = Molluscs.Cephalopoda.Coleoidea
 
-        cephalopod["order"] = Molluscs.Cephalopoda.Coleoidea
+        gastropod = mollusca.Gastropod()
 
-        gastropod = {}
         # gastropod_mass = '1 kg'
-        gastropod["mantle_mass"] = gastropod_mass
-
-        gastropod["order"] = Molluscs.Gastropoda.Pulmonata
+        gastropod.mantle_mass = gastropod_mass
+        gastropod.order = Molluscs.Gastropoda.Pulmonata
 
         # movie_path = 'path/to/gastropod/movie.mkv'
         with open(movie_path, "rb") as f:
-            gastropod["movie"] = f.read()
+            gastropod.movie = f.read()
 
-        request = {
-            'cephalopod': cephalopod,
-            'gastropod': gastropod,
-            'bivalve': "humboldt",
-        }
+        request = mollusca.CreateMolluscRequest(
+            cephalopod=cephalopod,
+            gastropod=gastropod,
+            bivalve="humboldt",
+        )
         ''',
         request=samplegen.FullRequest(
             request_list=[
@@ -252,9 +271,27 @@ def test_render_request_unflattened():
                                              single=None),
                 samplegen.TransformedRequest(base="bivalve",
                                              body=None,
-                                             single='"humboldt"'),
+                                             single=samplegen.AttributeRequestSetup(
+                                                 value='"humboldt"',
+                                             ),),
             ]
-        )
+        ),
+        module_name="mollusca",
+        request_type=common_types.DummyMessage(
+            fields={
+                "cephalopod": common_types.DummyField(
+                    name="cephalopod",
+                    type=common_types.DummyMessageTypePB(name="Cephalopod")
+                ),
+                "gastropod": common_types.DummyField(
+                    name="gastropod",
+                    type=common_types.DummyMessageTypePB(name="Gastropod")
+                )
+            },
+            ident=common_types.DummyIdent(name="CreateMolluscRequest")
+        ),
+        calling_form_enum=CallingForm,
+        calling_form=CallingForm.Request,
     )
 
 
@@ -265,7 +302,11 @@ def test_render_request_resource_name():
         {{ frags.render_request_setup(request) }}
         ''',
         '''
-        taxon = "kingdom/{kingdom}/phylum/{phylum}".format(kingdom="animalia", phylum=mollusca)
+        # Initialize request argument(s)
+        kingdom = "animalia"
+        phylum = mollusca
+        taxon = f"kingdom/{kingdom}/phylum/{phylum}"
+
         ''',
         request=samplegen.FullRequest(
             request_list=[
@@ -287,7 +328,7 @@ def test_render_request_resource_name():
                 ),
             ],
             flattenable=True
-        )
+        ),
     )
 
 
@@ -310,7 +351,7 @@ def test_render_print_args():
         {{ frags.render_print(["$resp %s %s", "$resp.squids", "$resp.clams"]) }}
         ''',
         '''
-        print("$resp {} {}".format(response.squids, response.clams))
+        print(f"$resp {response.squids} {response.clams}")
         '''
     )
 
@@ -411,7 +452,7 @@ def test_write_file():
                                     "contents": "$resp.photo"}) }}
         ''',
         '''
-        with open("specimen-{}".format(response.species), "wb") as f:
+        with open(f"specimen-{response.species}", "wb") as f:
             f.write(response.photo)
         '''
     )
@@ -427,7 +468,7 @@ def test_dispatch_write_file():
                                         "contents": "$resp.photo"}})}}
         ''',
         '''
-        with open("specimen-{}".format(response.species), "wb") as f:
+        with open(f"specimen-{response.species}", "wb") as f:
             f.write(response.photo)
 
         '''
@@ -442,7 +483,7 @@ def test_collection_loop():
         ''',
         '''
         for m in response.molluscs:
-            print("Mollusc: {}".format(m))
+            print(f"Mollusc: {m}")
 
 
         ''',
@@ -459,7 +500,7 @@ def test_dispatch_collection_loop():
         {{ frags.dispatch_statement(statement) }}''',
         '''
         for m in molluscs:
-            print("Mollusc: {}".format(m))
+            print(f"Mollusc: {m}")
 
 
 
@@ -478,7 +519,7 @@ def test_map_loop():
         }}''',
         '''
         for cls, example in response.molluscs.items():
-            print("A {} is a {}".format(example, cls))
+            print(f"A {example} is a {cls}")
 
 
         ''',
@@ -498,7 +539,7 @@ def test_map_loop_no_key():
         ''',
         '''
         for example in response.molluscs.values():
-            print("A {} is a mollusc".format(example))
+            print(f"A {example} is a mollusc")
 
 
         ''',
@@ -517,7 +558,7 @@ def test_map_loop_no_value():
         ''',
         '''
         for cls in response.molluscs.keys():
-            print("A {} is a mollusc".format(cls))
+            print(f"A {cls} is a mollusc")
 
 
         ''',
@@ -535,10 +576,10 @@ def test_dispatch_map_loop():
         ''',
         '''
         for cls, example in molluscs.items():
-            print("A {} is a {}".format(example, cls))
+            print(f"A {example} is a {cls}")
+
 
 
-        
         ''',
         statement={"loop": {"map": "molluscs",
                             "key": "cls",
@@ -583,14 +624,14 @@ def test_render_nested_loop_collection():
         for m in response.molluscs:
             for t in m.tentacles:
                 for s in t.suckers:
-                    print("Sucker: {}".format(s))
+                    print(f"Sucker: {s}")
+
+
+
+
+
 
 
-        
-        
-        
-        
-        
         """,
         statement=statement
     )
@@ -636,14 +677,14 @@ def test_render_nested_loop_map():
         for klass, orders in response.molluscs.items():
             for order, families in orders.items():
                 for family, ex in families.items():
-                    print("Example: {}".format(ex))
+                    print(f"Example: {ex}")
+
+
+
+
+
 
 
-        
-        
-        
-        
-        
         """,
         statement=statement
     )
@@ -694,7 +735,7 @@ def test_print_input_params():
 CALLING_FORM_TEMPLATE_TEST_STR = '''
         {% import "feature_fragments.j2" as frags %}
         {{ frags.render_calling_form("TEST_INVOCATION_TXT", calling_form,
-                                   calling_form_enum,
+                                   calling_form_enum, transport,
                                    [{"print": ["Test print statement"]}]) }}
         '''
 
@@ -702,70 +743,173 @@ CALLING_FORM_TEMPLATE_TEST_STR = '''
 def test_render_calling_form_request():
     check_template(CALLING_FORM_TEMPLATE_TEST_STR,
                    '''
+                   # Make the request
                    response = TEST_INVOCATION_TXT
+
+                   # Handle the response
                    print("Test print statement")
                    ''',
                    calling_form_enum=CallingForm,
-                   calling_form=CallingForm.Request)
+                   calling_form=CallingForm.Request,
+                   transport="grpc")
 
 
 def test_render_calling_form_paged_all():
     check_template(CALLING_FORM_TEMPLATE_TEST_STR,
                    '''
+                   # Make the request
                    page_result = TEST_INVOCATION_TXT
+
+                   # Handle the response
                    for response in page_result:
                        print("Test print statement")
                    ''',
                    calling_form_enum=CallingForm,
-                   calling_form=CallingForm.RequestPagedAll)
+                   calling_form=CallingForm.RequestPagedAll,
+                   transport="grpc")
+
+
+def test_render_calling_form_paged_all_async():
+    check_template(CALLING_FORM_TEMPLATE_TEST_STR,
+                   '''
+                   # Make the request
+                   page_result = TEST_INVOCATION_TXT
+
+                   # Handle the response
+                   async for response in page_result:
+                       print("Test print statement")
+                   ''',
+                   calling_form_enum=CallingForm,
+                   calling_form=CallingForm.RequestPagedAll,
+                   transport="grpc-async")
 
 
 def test_render_calling_form_paged():
     check_template(CALLING_FORM_TEMPLATE_TEST_STR,
                    '''
+                    # Make the request
                     page_result = TEST_INVOCATION_TXT
+
+                    # Handle the response
                     for page in page_result.pages():
                         for response in page:
                             print("Test print statement")
                    ''',
                    calling_form_enum=CallingForm,
-                   calling_form=CallingForm.RequestPaged)
+                   calling_form=CallingForm.RequestPaged,
+                   transport="grpc")
+
+
+def test_render_calling_form_paged_async():
+    check_template(CALLING_FORM_TEMPLATE_TEST_STR,
+                   '''
+                    # Make the request
+                    page_result = TEST_INVOCATION_TXT
+
+                    # Handle the response
+                    async for page in page_result.pages():
+                        for response in page:
+                            print("Test print statement")
+                   ''',
+                   calling_form_enum=CallingForm,
+                   calling_form=CallingForm.RequestPaged,
+                   transport="grpc-async")
 
 
 def test_render_calling_form_streaming_server():
     check_template(CALLING_FORM_TEMPLATE_TEST_STR,
                    '''
+                   # Make the request
                    stream = TEST_INVOCATION_TXT
+
+                   # Handle the response
                    for response in stream:
                        print("Test print statement")
                    ''',
                    calling_form_enum=CallingForm,
-                   calling_form=CallingForm.RequestStreamingServer)
+                   calling_form=CallingForm.RequestStreamingServer,
+                   transport="grpc")
+
+
+def test_render_calling_form_streaming_server_async():
+    check_template(CALLING_FORM_TEMPLATE_TEST_STR,
+                   '''
+                   # Make the request
+                   stream = TEST_INVOCATION_TXT
+
+                   # Handle the response
+                   async for response in stream:
+                       print("Test print statement")
+                   ''',
+                   calling_form_enum=CallingForm,
+                   calling_form=CallingForm.RequestStreamingServer,
+                   transport="grpc-async")
 
 
 def test_render_calling_form_streaming_bidi():
     check_template(CALLING_FORM_TEMPLATE_TEST_STR,
                    '''
+                   # Make the request
                    stream = TEST_INVOCATION_TXT
+
+                   # Handle the response
                    for response in stream:
                        print("Test print statement")
                    ''',
                    calling_form_enum=CallingForm,
-                   calling_form=CallingForm.RequestStreamingBidi)
+                   calling_form=CallingForm.RequestStreamingBidi,
+                   transport="grpc")
+
+
+def test_render_calling_form_streaming_bidi_async():
+    check_template(CALLING_FORM_TEMPLATE_TEST_STR,
+                   '''
+                   # Make the request
+                   stream = TEST_INVOCATION_TXT
+
+                   # Handle the response
+                   async for response in stream:
+                       print("Test print statement")
+                   ''',
+                   calling_form_enum=CallingForm,
+                   calling_form=CallingForm.RequestStreamingBidi,
+                   transport="grpc-async")
 
 
 def test_render_calling_form_longrunning():
     check_template(CALLING_FORM_TEMPLATE_TEST_STR,
                    '''
+                   # Make the request
                    operation = TEST_INVOCATION_TXT
 
                    print("Waiting for operation to complete...")
 
                    response = operation.result()
+
+                   # Handle the response
                    print("Test print statement")
                    ''',
                    calling_form_enum=CallingForm,
-                   calling_form=CallingForm.LongRunningRequestPromise)
+                   calling_form=CallingForm.LongRunningRequestPromise,
+                   transport="grpc")
+
+
+def test_render_calling_form_longrunning_async():
+    check_template(CALLING_FORM_TEMPLATE_TEST_STR,
+                   '''
+                   # Make the request
+                   operation = TEST_INVOCATION_TXT
+
+                   print("Waiting for operation to complete...")
+
+                   response = await operation.result()
+
+                   # Handle the response
+                   print("Test print statement")
+                   ''',
+                   calling_form_enum=CallingForm,
+                   calling_form=CallingForm.LongRunningRequestPromise,
+                   transport="grpc-async")
 
 
 def test_render_method_call_basic():
@@ -773,7 +917,7 @@ def test_render_method_call_basic():
         '''
         {% import "feature_fragments.j2" as frags %}
         {{ frags.render_method_call({"rpc": "CategorizeMollusc", "request": request},
-                                  calling_form, calling_form_enum) }}
+                                  calling_form, calling_form_enum, transport) }}
         ''',
         '''
         client.categorize_mollusc(request=request)
@@ -792,7 +936,66 @@ def test_render_method_call_basic():
             ],
         ),
         calling_form_enum=CallingForm,
-        calling_form=CallingForm.Request
+        calling_form=CallingForm.Request,
+        transport="grpc"
+    )
+
+
+def test_render_method_call_basic_async():
+    check_template(
+        '''
+        {% import "feature_fragments.j2" as frags %}
+        {{ frags.render_method_call({"rpc": "CategorizeMollusc", "request": request},
+                                  calling_form, calling_form_enum, transport) }}
+        ''',
+        '''
+        await client.categorize_mollusc(request=request)
+        ''',
+        request=samplegen.FullRequest(
+            request_list=[
+                samplegen.TransformedRequest(base="video",
+                                             body=True,
+                                             single=None),
+                samplegen.TransformedRequest(base="audio",
+                                             body=True,
+                                             single=None),
+                samplegen.TransformedRequest(base="guess",
+                                             body=True,
+                                             single=None)
+            ],
+        ),
+        calling_form_enum=CallingForm,
+        calling_form=CallingForm.Request,
+        transport="grpc-async"
+    )
+
+
+def test_render_method_call_basic_async():
+    check_template(
+        '''
+        {% import "feature_fragments.j2" as frags %}
+        {{ frags.render_method_call({"rpc": "CategorizeMollusc", "request": request},
+                                  calling_form, calling_form_enum, transport) }}
+        ''',
+        '''
+        await client.categorize_mollusc(request=request)
+        ''',
+        request=samplegen.FullRequest(
+            request_list=[
+                samplegen.TransformedRequest(base="video",
+                                             body=True,
+                                             single=None),
+                samplegen.TransformedRequest(base="audio",
+                                             body=True,
+                                             single=None),
+                samplegen.TransformedRequest(base="guess",
+                                             body=True,
+                                             single=None)
+            ],
+        ),
+        calling_form_enum=CallingForm,
+        calling_form=CallingForm.Request,
+        transport="grpc-async"
     )
 
 
@@ -801,7 +1004,7 @@ def test_render_method_call_basic_flattenable():
         '''
         {% import "feature_fragments.j2" as frags %}
         {{ frags.render_method_call({"rpc": "CategorizeMollusc", "request": request},
-                                  calling_form, calling_form_enum) }}
+                                  calling_form, calling_form_enum, transport) }}
         ''',
         '''
         client.categorize_mollusc(video=video, audio=audio, guess=guess)
@@ -821,7 +1024,8 @@ def test_render_method_call_basic_flattenable():
             flattenable=True,
         ),
         calling_form_enum=CallingForm,
-        calling_form=CallingForm.Request
+        calling_form=CallingForm.Request,
+        transport="grpc"
     )
 
 
@@ -830,10 +1034,10 @@ def test_render_method_call_bidi():
         '''
         {% import "feature_fragments.j2" as frags %}
         {{ frags.render_method_call({"rpc": "CategorizeMollusc", "request": request},
-                                  calling_form, calling_form_enum) }}
+                                  calling_form, calling_form_enum, transport) }}
         ''',
         '''
-        client.categorize_mollusc([video])
+        client.categorize_mollusc(requests=request_generator())
         ''',
         request=samplegen.FullRequest(
             request_list=[
@@ -845,7 +1049,33 @@ def test_render_method_call_bidi():
             ]
         ),
         calling_form_enum=CallingForm,
-        calling_form=CallingForm.RequestStreamingBidi
+        calling_form=CallingForm.RequestStreamingBidi,
+        transport="grpc",
+    )
+
+
+def test_render_method_call_bidi_async():
+    check_template(
+        '''
+        {% import "feature_fragments.j2" as frags %}
+        {{ frags.render_method_call({"rpc": "CategorizeMollusc", "request": request},
+                                  calling_form, calling_form_enum, transport) }}
+        ''',
+        '''
+        await client.categorize_mollusc(requests=request_generator())
+        ''',
+        request=samplegen.FullRequest(
+            request_list=[
+                samplegen.TransformedRequest(
+                    base="video",
+                    body=True,
+                    single=None
+                )
+            ]
+        ),
+        calling_form_enum=CallingForm,
+        calling_form=CallingForm.RequestStreamingBidi,
+        transport="grpc-async",
     )
 
 
@@ -854,10 +1084,10 @@ def test_render_method_call_client():
         '''
         {% import "feature_fragments.j2" as frags %}
         {{ frags.render_method_call({"rpc": "CategorizeMollusc", "request": request},
-        calling_form, calling_form_enum) }}
+        calling_form, calling_form_enum, transport) }}
         ''',
         '''
-        client.categorize_mollusc([video])
+        client.categorize_mollusc(requests=request_generator())
         ''',
         request=samplegen.FullRequest(
             request_list=[
@@ -869,7 +1099,33 @@ def test_render_method_call_client():
             ]
         ),
         calling_form_enum=CallingForm,
-        calling_form=CallingForm.RequestStreamingClient
+        calling_form=CallingForm.RequestStreamingClient,
+        transport="grpc",
+    )
+
+
+def test_render_method_call_client_async():
+    check_template(
+        '''
+        {% import "feature_fragments.j2" as frags %}
+        {{ frags.render_method_call({"rpc": "CategorizeMollusc", "request": request},
+        calling_form, calling_form_enum, transport) }}
+        ''',
+        '''
+        await client.categorize_mollusc(requests=request_generator())
+        ''',
+        request=samplegen.FullRequest(
+            request_list=[
+                samplegen.TransformedRequest(
+                    base="video",
+                    body=True,
+                    single=None
+                )
+            ]
+        ),
+        calling_form_enum=CallingForm,
+        calling_form=CallingForm.RequestStreamingClient,
+        transport="grpc-async",
     )
 
 
